@@ -2,6 +2,7 @@ import numpy as np
 import pandas as pd
 import pytest
 
+from woodwork.logical_types import Boolean, Datetime, Integer, NaturalLanguage
 from woodwork.utils import import_or_none
 
 
@@ -24,18 +25,12 @@ def spark_session():
             .master("local[*]") \
             .config("spark.driver.extraJavaOptions", "-Dio.netty.tryReflectionSetAccessible=True") \
             .config("spark.sql.shuffle.partitions", "2") \
+            .config("spark.driver.bindAddress", "127.0.0.1") \
             .getOrCreate()
 
         return spark
 
 
-@pytest.fixture(params=[('sample_df_pandas', 'sample_series_pandas'),
-                        ('sample_df_dask', 'sample_series_dask'),
-                        ('sample_df_koalas', 'sample_series_koalas')])
-def sample_combos(request):
-    return (request.getfixturevalue(request.param[0]), request.getfixturevalue(request.param[1]))
-
-
 @pytest.fixture(params=['sample_df_pandas', 'sample_df_dask', 'sample_df_koalas'])
 def sample_df(request):
     return request.getfixturevalue(request.param)
@@ -48,9 +43,9 @@ def sample_df_pandas():
         'full_name': ['Mr. John Doe', 'Doe, Mrs. Jane', 'James Brown', 'Ms. Paige Turner'],
         'email': ['john.smith@example.com', np.nan, 'team@featuretools.com', 'junk@example.com'],
         'phone_number': ['5555555555', '555-555-5555', '1-(555)-555-5555', '555-555-5555'],
-        'age': [33, 25, 33, 57],
+        'age': pd.Series([pd.NA, 33, 33, 57], dtype='Int64'),
         'signup_date': [pd.to_datetime('2020-09-01')] * 4,
-        'is_registered': [True, False, True, True],
+        'is_registered': pd.Series([pd.NA, False, True, True], dtype='boolean'),
     })
 
 
@@ -103,7 +98,7 @@ def sample_series(request):
 
 @pytest.fixture()
 def sample_series_pandas():
-    return pd.Series(['a', 'b', 'c', 'a'], name='sample_series').astype('object')
+    return pd.Series(['a', 'b', 'c', 'a'], name='sample_series').astype('category')
 
 
 @pytest.fixture()
@@ -115,7 +110,7 @@ def sample_series_dask(sample_series_pandas):
 @pytest.fixture()
 def sample_series_koalas(sample_series_pandas):
     ks = pytest.importorskip('databricks.koalas', reason='Koalas not installed, skipping')
-    return ks.from_pandas(sample_series_pandas)
+    return ks.from_pandas(sample_series_pandas.astype('string'))
 
 
 @pytest.fixture(params=['sample_datetime_series_pandas', 'sample_datetime_series_dask', 'sample_datetime_series_koalas'])
@@ -173,8 +168,8 @@ def time_index_df(request):
 def numeric_time_index_df_pandas():
     return pd.DataFrame({
         'floats': pd.Series([1, 2, 3, 4], dtype='float'),
-        'ints': pd.Series([1, -2, 3, 4], dtype='Int64'),
-        'with_null': pd.Series([1, 2, pd.NA, 4], dtype='Int64'),
+        'ints': pd.Series([1, -2, 3, 4], dtype='int64'),
+        'with_null': pd.Series([1, pd.NA, 3, 4], dtype='Int64'),
     })
 
 
@@ -187,8 +182,6 @@ def numeric_time_index_df_dask(numeric_time_index_df_pandas):
 @pytest.fixture()
 def numeric_time_index_df_koalas(numeric_time_index_df_pandas):
     ks = pytest.importorskip('databricks.koalas', reason='Koalas not installed, skipping')
-    numeric_time_index_df_pandas['ints'] = numeric_time_index_df_pandas['ints'].astype('int64')
-    numeric_time_index_df_pandas['with_null'] = numeric_time_index_df_pandas['ints'].astype('float')
     return ks.from_pandas(numeric_time_index_df_pandas)
 
 
@@ -200,7 +193,7 @@ def numeric_time_index_df(request):
 @pytest.fixture()
 def describe_df_pandas():
     index_data = [0, 1, 2, 3, 4, 5, 6, 7]
-    boolean_data = [True, False, True, True, False, True, False, True]
+    boolean_data = [True, False, True, True, False, True, False, None]
     category_data = ['red', 'blue', 'red', np.nan, 'red', 'blue', 'red', 'yellow']
     datetime_data = pd.to_datetime(['2020-01-01',
                                     '2020-02-01',
@@ -458,6 +451,7 @@ def latlong_df(request):
 @pytest.fixture
 def pandas_latlongs():
     return [
+        pd.Series([(1.0, 2.0), (3.0, 4.0)]),
         pd.Series([('1', '2'), ('3', '4')]),
         pd.Series([['1', '2'], ['3', '4']]),
         pd.Series([(1, 2), (3, 4)]),
@@ -506,3 +500,79 @@ def falsy_names_df_koalas(falsy_names_df_pandas):
 @pytest.fixture(params=['falsy_names_df_pandas', 'falsy_names_df_dask', 'falsy_names_df_koalas'])
 def falsy_names_df(request):
     return request.getfixturevalue(request.param)
+
+
+@pytest.fixture()
+def sample_column_names():
+    return ['id',
+            'full_name',
+            'email',
+            'phone_number',
+            'age',
+            'signup_date',
+            'is_registered']
+
+
+@pytest.fixture()
+def sample_inferred_logical_types():
+    return {'id': Integer,
+            'full_name': NaturalLanguage,
+            'email': NaturalLanguage,
+            'phone_number': NaturalLanguage,
+            'age': Integer,
+            'signup_date': Datetime,
+            'is_registered': Boolean}
+
+
+@pytest.fixture()
+def serialize_df_pandas():
+    df = pd.DataFrame({
+        'id': [0, 1, 2],
+        'cat_int': [1, 2, 1],
+        'ord_int': [1, 2, 1],
+        'cat_float': [1.0, 2.0, 1.0],
+        'ord_float': [1.0, 2.0, 1.0],
+        'cat_bool': [True, False, True],
+        'ord_bool': [True, False, True],
+    })
+    return df
+
+
+@pytest.fixture()
+def serialize_df_dask(serialize_df_pandas):
+    dd = pytest.importorskip('dask.dataframe', reason='Dask not installed, skipping')
+    return dd.from_pandas(serialize_df_pandas, npartitions=2)
+
+
+@pytest.fixture()
+def serialize_df_koalas(serialize_df_pandas):
+    ks = pytest.importorskip('databricks.koalas', reason='Koalas not installed, skipping')
+    return ks.from_pandas(serialize_df_pandas)
+
+
+@pytest.fixture(params=['serialize_df_pandas', 'serialize_df_dask', 'serialize_df_koalas'])
+def serialize_df(request):
+    return request.getfixturevalue(request.param)
+
+
+@pytest.fixture()
+def pandas_datetimes():
+    return [
+        pd.Series(['3/11/2000', '3/12/2000', '3/13/2000', '3/14/2000']),
+        pd.Series(['3/11/2000', np.nan, '3/13/2000', '3/14/2000']),
+    ]
+
+
+@pytest.fixture()
+def dask_datetimes(pandas_datetimes):
+    return [pd_to_dask(series) for series in pandas_datetimes]
+
+
+@pytest.fixture()
+def koalas_datetimes(pandas_datetimes):
+    return [pd_to_koalas(series) for series in pandas_datetimes]
+
+
+@pytest.fixture(params=['pandas_datetimes', 'dask_datetimes', 'koalas_datetimes'])
+def datetimes(request):
+    return request.getfixturevalue(request.param)
