@@ -1,32 +1,39 @@
 from .inference_functions import (
     boolean_func,
+    boolean_nullable_func,
     categorical_func,
     datetime_func,
     double_func,
     integer_func,
+    integer_nullable_func,
     timedelta_func
 )
 
 from woodwork.logical_types import (
     URL,
+    Address,
+    Age,
+    AgeNullable,
     Boolean,
+    BooleanNullable,
     Categorical,
     CountryCode,
     Datetime,
     Double,
     EmailAddress,
     Filepath,
-    FullName,
     Integer,
+    IntegerNullable,
     IPAddress,
     LatLong,
     LogicalType,
     NaturalLanguage,
     Ordinal,
+    PersonFullName,
     PhoneNumber,
+    PostalCode,
     SubRegionCode,
-    Timedelta,
-    ZIPCode
+    Timedelta
 )
 from woodwork.utils import import_or_none
 
@@ -34,35 +41,45 @@ dd = import_or_none('dask.dataframe')
 ks = import_or_none('databricks.koalas')
 
 DEFAULT_INFERENCE_FUNCTIONS = {
+    Address: None,
+    Age: None,
+    AgeNullable: None,
     Boolean: boolean_func,
+    BooleanNullable: boolean_nullable_func,
     Categorical: categorical_func,
     CountryCode: None,
     Datetime: datetime_func,
     Double: double_func,
     EmailAddress: None,
     Filepath: None,
-    FullName: None,
+    PersonFullName: None,
     Integer: integer_func,
+    IntegerNullable: integer_nullable_func,
     IPAddress: None,
     LatLong: None,
     NaturalLanguage: None,
     Ordinal: None,
     PhoneNumber: None,
+    PostalCode: None,
     SubRegionCode: None,
     Timedelta: timedelta_func,
-    URL: None,
-    ZIPCode: None
+    URL: None
 }
 
 # (ParentType, ChildType)
 DEFAULT_RELATIONSHIPS = [
+    (BooleanNullable, Boolean),
     (Categorical, CountryCode),
     (Categorical, Ordinal),
+    (Categorical, PostalCode),
     (Categorical, SubRegionCode),
-    (Categorical, ZIPCode),
+    (Integer, Age),
+    (IntegerNullable, AgeNullable),
+    (IntegerNullable, Integer),
+    (NaturalLanguage, Address),
     (NaturalLanguage, EmailAddress),
     (NaturalLanguage, Filepath),
-    (NaturalLanguage, FullName),
+    (NaturalLanguage, PersonFullName),
     (NaturalLanguage, IPAddress),
     (NaturalLanguage, PhoneNumber),
     (NaturalLanguage, URL),
@@ -269,10 +286,10 @@ class TypeSystem(object):
 
         if len(type_matches) == 0:
             # If no matches, set type to default type (NaturalLanguage)
-            return self.default_type
+            logical_type = self.default_type
         elif len(type_matches) == 1:
             # If we match only one type, return it
-            return type_matches[0]
+            logical_type = type_matches[0]
         else:
             # If multiple matches, get the most specific one. If multiple
             # matches have the same level of specificity, the first
@@ -284,7 +301,9 @@ class TypeSystem(object):
                 if ltype_depth > best_depth:
                     best_match = logical_type
                     best_depth = ltype_depth
-            return best_match
+            logical_type = best_match
+
+        return logical_type()
 
     def _get_logical_types(self):
         """Returns a dictionary of logical type name strings and logical type classes"""
